/**
 * 
 */
package com.cybozu.garoon3.address;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.axiom.om.OMElement;
import org.apache.xml.dtm.ref.DTMNodeList;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import au.com.bytecode.opencsv.CSVReader;

import com.cybozu.garoon3.usecase.util.CSVUtil;

/**
 * アドレス帳、アドレスカードを参照するとき、共通な操作関数
 * 
 * @author $api_author Garoon Team@Cybozu$
 * @version $api_version ver 1.0.0$
 */
public class AddressUtility {
    private static final String ADDRESS_CUSTOM_NODE = "book:custom";    
    private static final String CARD_CREATOR_NODE = "card:creator";
    private static final String CARD_MODIFIER_NODE = "card:modifier";
    private static final String CARD_PERSONAL_NAME_NODE = "card:personal_name";
    private static final String CARD_PERSONAL_READING_NODE = "card:personal_reading";
    private static final String CARD_CUSTOM_NODE = "card:custom";
    
    private static final String ADDRESS_ROUTE_NODE = "book:route";    
    private static final String CARD_ROUTE_NODE = "card:route";
    
    private static final String ADDRESS_ROUTE_DETAIL_NODE = "book:route_detail";    
    
    private static final String ROUTE_DETAIL_CODE[] = {"path", "time", "fare"};
    private static final String ROUTE_DETAIL_DISPLAY[] = {"経路", "所要時間", "運賃"};
    private static int ROUTE_DETAIL_NUM_FIELDS = 3;
    
    private static final String ACTION_REMOVE = "remove";
    
    /**
     * アドレス帳の XML データからアドレスカードIDのリストを取得します。
     * 
     * @param input 入力するアドレス帳データ
     * @param responseName SOAP レスポンスの API 名
     * @return アドレスカードIDの配列
     */
    public static List<Integer> getCardIdsFromXML(OMElement input, String responseName) {
        //XMLDisplayUtility.printSmartXML(input);
		Document document = null;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(new InputSource(new StringReader(input.toString())));
		} catch(IOException e) {
			return Collections.emptyList();
		} catch (SAXException e) {
			return Collections.emptyList();
		} catch (ParserConfigurationException e) {
			return Collections.emptyList();
		}

		DTMNodeList node = null;

		try{
	        XPath xPath = XPathFactory.newInstance().newXPath();
	        XPathExpression xPathExpression = xPath.compile(responseName + "/returns/book/cards/card");
	        node = (DTMNodeList)xPathExpression.evaluate(document, XPathConstants.NODESET);
		}catch(XPathExpressionException e) {
			return Collections.emptyList();
		}

		List<Integer> cardIds = new ArrayList<Integer>();
        for (int i = 0; i < node.getLength(); i++){
        	try{
                XPath xPath = XPathFactory.newInstance().newXPath();
                XPathExpression xPathExpression = xPath.compile("@id");
	            String val = (String)xPathExpression.evaluate(node.item(i), XPathConstants.STRING);
	            cardIds.add( Integer.valueOf(val) );
        	}catch(XPathExpressionException e){
        		continue;
        	}
        }

        return cardIds;
    }
    
    /**
     * XML データからアドレスカードのフォーマットを取得します。
     * 
     * @param attributionsNode アドレスカードフォーマットを含む XML データ
     * @return アドレスカードのフォーマット
     */
    private static AddressCardFormat getAttributions(Node attributionsNode) {
        return getAttributions(attributionsNode, false);
    }

    /**
     * XML データからアドレスカードのフォーマットを取得します。
     * 
     * @param attributionsNode アドレスカードフォーマットを含む XML データ
     * @param customNode 一般項目かカスタマイズ項目かを指定
     * @return アドレスカードのフォーマット
     */
    private static AddressCardFormat getAttributions(Node attributionsNode, boolean customNode) {
        if (attributionsNode == null)
        	return null;
        
        NodeList attributions = attributionsNode.getChildNodes();
        XPath xPath =  XPathFactory.newInstance().newXPath();
        
        XPathExpression xPathFieldCode = null;
        XPathExpression xPathDisplayName = null;
        try {
        	xPathFieldCode = xPath.compile("@field_code");
        	xPathDisplayName = xPath.compile("@name");
        }catch(XPathExpressionException e) {
        	return null;
        }
        
        AddressCardFormat cardFormat = new AddressCardFormat();

        for (int i = 0; i < attributions.getLength(); i++){
            Node data = attributions.item(i);
            if (data.getNodeType() != Node.ELEMENT_NODE)
            	continue;
            
            String nodeName = data.getNodeName();
            if (ADDRESS_CUSTOM_NODE.compareTo(nodeName) == 0) {
                AddressCardFormat customFields = getAttributions(data, true);
                cardFormat.merge(customFields);
                continue;
            }
            
            String displayName = null;
			try {
				displayName = (String)xPathDisplayName.evaluate(data, XPathConstants.STRING);
			} catch (XPathExpressionException e) {
				continue;
			}
            
            String localNodeName = nodeName.replaceFirst("book:", "");
            
            if (!customNode){
                AddressCardAttribution attr = new AddressCardAttribution(localNodeName, displayName);
                
                // book:route_detail を定義したが、実際  card にそれを使っていないので、無視します
                if (nodeName.compareTo(ADDRESS_ROUTE_DETAIL_NODE) == 0) continue;
                
                /**
                 *  "card:route" の中に "card:path", "card:time", "card:fare" があるが、
                 *  ブックフォーマットのミスで、その定義が入っていません。なので、手動で追加します。
                 */
                if (nodeName.compareTo(ADDRESS_ROUTE_NODE) == 0){
                    for (int j = 0; j < ROUTE_DETAIL_NUM_FIELDS; j++){
                        String subFieldCode = ROUTE_DETAIL_CODE[j];
                        String subFieldName = displayName + "・" + ROUTE_DETAIL_DISPLAY[j];
                        attr = new AddressCardAttribution(subFieldCode, subFieldName);
                        cardFormat.addAttribution(subFieldCode, attr);
                    }
                }
                else {
                    cardFormat.addAttribution(localNodeName, attr);
                }
            }
            else {
                String fieldCode = null;
				try {
					fieldCode = (String)xPathFieldCode.evaluate(data, XPathConstants.STRING);
				} catch (XPathExpressionException e) {
					continue;
				}

				AddressCardAttribution attr = new AddressCardAttribution(fieldCode, displayName);
				cardFormat.addAttribution(fieldCode, attr);
            }
        }
        return cardFormat;
    }

    /**
     * アドレスカードのフォーマットを取得します。
     * 
     * @param input 入力するアドレスカードフォーマットの XML 表現
     * @param responseName SOAP レスポンスの API 名
     * @return アドレスカードのフォーマット
     */
    public static AddressCardFormat getCardFormatFromXML(OMElement input, String responseName) {
		Document document = null;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(new InputSource(new StringReader(input.toString())));
		} catch(IOException e){
			return null;
		} catch (SAXException e) {
			return null;
		} catch (ParserConfigurationException e) {
			return null;
		}

        XPath xPath =  XPathFactory.newInstance().newXPath();
		try {
			XPathExpression xPathExpression = xPath.compile(responseName + "/returns/book/form");
	        Node attrNode = (Node)xPathExpression.evaluate(document, XPathConstants.NODE);
	        return getAttributions(attrNode);
		} catch (XPathExpressionException e) {
			return null;
		}
    }
    
    /**
     * XML のテキストデータを繋いで一つの文字列に格納します。
     * 
     * @param name_node　XML のノード
     * @return テキスト部分の文字列
     */
    private static String concatFields(Node nameNode){
        NodeList nameParts = nameNode.getChildNodes();
        
        String result = "";
        for (int i = 0; i < nameParts.getLength(); i++){
            Node data = nameParts.item(i);
            if (data.getNodeType() == Node.TEXT_NODE)
            	continue;
            
            if (result.length() > 0)
            	result = result + " ";
            
            result = result + data.getTextContent();
        }
        
        return result;
    }
    
    /**
     * カスタム項目の情報を取得します。
     * 
     * @param customNode カスタム項目の XML データ
     * @param cardData カスタマ情報を格納する
     */
    private static void extractCustomField(Node customNode, AddressCardData cardData) {
        NodeList nameParts = customNode.getChildNodes();
        
        for (int i = 0; i < nameParts.getLength(); i++){
            Node data = nameParts.item(i);
            if (data.getNodeType() == Node.TEXT_NODE)
            	continue;
            
            XPath xPath =  XPathFactory.newInstance().newXPath();
            try{
	            XPathExpression xPathFieldCode = xPath.compile("@field_code");
	            String field_code = (String)xPathFieldCode.evaluate(data, XPathConstants.STRING);
	            String dataText = data.getTextContent();
	            cardData.setCardData(field_code, dataText);
            }catch(XPathExpressionException e) {
            	continue;
            }
        }
    }
    
    /**
     * 路線情報の XML 形式から詳細項目を取りだす
     * 
     * @param routeNode 路線情報の XML データ
     * @param cardData カード情報のコンテナー
     */
    private static void extractRouteField(Node routeNode, AddressCardData cardData) {
        NodeList nameParts = routeNode.getChildNodes();

        for (int i = 0; i < nameParts.getLength(); i++){
            Node data = nameParts.item(i);
            if (data.getNodeType() == Node.TEXT_NODE)
                continue;
            
            String field_code = data.getNodeName();
            field_code = field_code.replaceFirst("card:", "");
            String dataText = data.getTextContent();
            cardData.setCardData(field_code, dataText);
        }
    }
    
    /**
     * XML データからアドレスカードを取得して、アドレス帳に格納します。
     * 
     * @param input アドレスカードの XML データ
     * @param book アドレスカードを保持するアドレス帳
     * @param response_name SOAP レスポンスのアクション名
     */
    public static void getCardDataFromXMLToBook(OMElement input, AddressBook book, String response_name) {
        //XMLDisplayUtility.printSmartXML(input);
		Document document = null;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(new InputSource(new StringReader(input.toString())));
		} catch(IOException e){
			return;
		} catch (SAXException e) {
			return;
		} catch (ParserConfigurationException e) {
			return;
		}
        
        NodeList cardListNode = null;
        try{
        	XPath xPath =  XPathFactory.newInstance().newXPath();
	        XPathExpression xPathExpression = xPath.compile(response_name + "/returns/card");
	        cardListNode = (NodeList)xPathExpression.evaluate(document, XPathConstants.NODESET);
        }catch(XPathExpressionException e) {
        	return;
        }
        
        for (int i = 0; i < cardListNode.getLength(); i++){
            //System.out.println("Card " + i);
            Node cardNode = cardListNode.item(i);
            
            String cardIdStr = null;
            try {
            	XPath xPath =  XPathFactory.newInstance().newXPath();
				XPathExpression xPathExpression = xPath.compile("@id");
	            cardIdStr = (String)xPathExpression.evaluate(cardNode, XPathConstants.STRING);
			} catch (XPathExpressionException e) {
				return;
			}
            
            int cardId = Integer.parseInt( cardIdStr );
            
            NodeList cardDetails = cardNode.getChildNodes();
            
            AddressCardData cardData = new AddressCardData();            
            for (int j = 0; j < cardDetails.getLength(); j++){
                Node fieldData = cardDetails.item(j);
                if (fieldData.getNodeType() == Node.TEXT_NODE)
                	continue;

                String nodeName = fieldData.getNodeName();
                
                // Skip creator and modifier
                if (nodeName.compareTo(CARD_CREATOR_NODE) == 0)
                	continue;
                if (nodeName.compareTo(CARD_MODIFIER_NODE) == 0)
                	continue;
                
                if (nodeName.compareTo(CARD_PERSONAL_NAME_NODE) == 0  || nodeName.compareTo(CARD_PERSONAL_READING_NODE) == 0) {
                    String fullText = concatFields(fieldData);
                    String fieldName = nodeName.replaceFirst("card:", "");
                    cardData.setCardData(fieldName, fullText);
                    continue;
                }
                
                if (nodeName.compareTo(CARD_CUSTOM_NODE) == 0){
                    extractCustomField(fieldData, cardData);
                    continue;
                }
                
                if (nodeName.compareTo(CARD_ROUTE_NODE) == 0){
                    extractRouteField(fieldData, cardData);
                    continue;
                }
                
                String fieldName = nodeName.replaceFirst("card:", "");
                String fieldText = fieldData.getTextContent();
                cardData.setCardData(fieldName, fieldText);
            }
            
            book.insertAddressCard(cardId, cardData);
            //cardData.printCardDetails();
        }
    }
    
    /**
     * CSV ファイルから個人アドレス帳や共有アドレス帳のID一覧を取得します。
     * 
     * @param inputFileName 入力する CSV ファイル
     * @param idName CSV ファイルにあるカラム名
     * @return 個人アドレス帳や共有アドレス帳のID一覧
     */
    public static List<Integer> getIDList( String inputFileName, String idName )
    {
        //Read input csv file
        CSVReader reader;
        try {
            reader = new CSVReader( new FileReader( inputFileName ) );
        }
        catch (FileNotFoundException e){
            return null;
        }
        List<String> idStrList = CSVUtil.getColumn(reader, idName);

        //Parse String to Integer
        List<Integer> idList = new ArrayList<Integer>();
        for(String id : idStrList){
            idList.add( Integer.valueOf(id) );
        }
        
        return idList;        
    }
    
    /**
     * アドレス帳のバージョン情報から個人アドレス帳 ID を取得します。
     * @param input アドレス帳のバージョン情報
     * @return 個人アドレス帳 ID
     */
    public static List<Integer> getPersonalAddressBook(OMElement input){
        Document document = null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(new InputSource(new StringReader(input.toString())));
        } catch(IOException e) {
            return Collections.emptyList();
        } catch (SAXException e) {
            return Collections.emptyList();
        } catch (ParserConfigurationException e) {
            return Collections.emptyList();
        }
        
        NodeList bookListNode = null;
        String response_name = "AddressGetPersonalBookVersionsResponse";
        try{
            XPath xPath =  XPathFactory.newInstance().newXPath();
            XPathExpression xPathExpression = xPath.compile(response_name + "/returns/book_item");
            bookListNode = (NodeList)xPathExpression.evaluate(document, XPathConstants.NODESET);
        }catch(XPathExpressionException e) {
            return Collections.emptyList();
        }
        
        List<Integer> book_ids = new ArrayList<Integer>();
        for (int i = 0; i < bookListNode.getLength(); i++){
            try{
                XPath xPath = XPathFactory.newInstance().newXPath();
                XPathExpression xPathExpression = xPath.compile("@operation");
                String book_operation = (String)xPathExpression.evaluate(bookListNode.item(i), XPathConstants.STRING);
                if (book_operation.compareTo(ACTION_REMOVE) == 0) continue;
                
                xPathExpression = xPath.compile("@id");
                String book_id = (String)xPathExpression.evaluate(bookListNode.item(i), XPathConstants.STRING);
                //System.out.println(book_id);
                book_ids.add( Integer.valueOf(book_id) );
            }catch(XPathExpressionException e){
                continue;
            }
        }

        return book_ids;
    }
}
