package com.cybozu.garoon3.admin;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;

import com.cybozu.garoon3.common.APIType;
import com.cybozu.garoon3.common.Action;

/**
 * ユーザーの追加を行います。<br />
 * 一回のAPIで追加できるユーザーは一人です。<br />
 * 複数のユーザーを追加したい場合はこのクラスを複数回用いる必要があります。<br />
 * <br />
 * ユーザー情報はログイン名、表示名、パスワードの三つで構成されます。<br />
 * ログイン名と表示名を省略することはできません。<br />
 * 
 * @author $api_author Garoon Team@Cybozu$
 * @version $api_version ver 1.0.0$
 */
public class AdminAddUserAccount implements Action {
    private String loginName;
    private String displayName;
    private String password;
    
    /**
     * ログイン名を設定します。
     * null または空文字は設定不可。
	 *
     * @param loginName ログイン名
     */
    public void setLoginName(String loginName) { this.loginName = loginName; }

    /**
     * 表示名を設定します。
     * null または空文字は設定不可。
     * 
     * @param displayName 表示名
     */
    public void setDisplayName(String displayName) { this.displayName = displayName; }

    /**
     * パスワードを設定します。
     * パスワードは空白文字のみで構成することはできません。
     * 設定された値が null または String#trim 後の長さが 0 の場合、パスワードは無視されます。
	 *
     * @param password パスワード
     * @see java.lang.String#trim()
     */
    public void setPassword(String password) { this.password = password; }

    /**
     * このクラスを利用して追加するユーザーのログイン名と表示名を追加します。
     * 
     * @param loginName ログイン名
     * @param displayName 表示名
     * @see #setLoginName(String)
     * @see #setDisplayName(String)
     */
    public AdminAddUserAccount( String loginName, String displayName)
    {
        this( loginName, displayName, "");
    }
    
    /**
     * このクラスを利用して追加するユーザーのログイン名と表示名、パスワードを追加します。
     * 
     * @param loginName ログイン名
     * @param displayName 表示名
     * @param password パスワード
     * @see #setLoginName(String)
     * @see #setDisplayName(String)
     * @see #setPassword(String)
     */
    public AdminAddUserAccount(String loginName, String displayName, String password)
    {
        this.loginName = loginName;
        this.displayName = displayName;
        this.password = password;
    }
    
    @Override
    public String getActionName() {
        return "AdminAddUserAccount";
    }

    @Override
    public OMElement getParameters() {
        OMFactory omFactory = OMAbstractFactory.getOMFactory();
        OMElement parameters = omFactory.createOMElement("parameters", null);

        OMElement loginNameNode = omFactory.createOMElement("login_name", null);
        loginNameNode.setText( this.loginName );
        
        OMElement displayNameNode = omFactory.createOMElement("display_name", null);
        displayNameNode.setText( this.displayName );
        
        OMElement passwordNode = omFactory.createOMElement("password_raw", null);
        passwordNode.setText( this.password );
        
        parameters.addChild( loginNameNode );
        parameters.addChild( displayNameNode );
        
        if(password != null && password.trim().length() > 0)
            parameters.addChild( passwordNode );

        return parameters;
    }

    @Override
    public APIType getAPIType() {
        return APIType.ADMIN;
    }
}
