package com.cybozu.garoon3.common;

import java.net.URI;

import java.net.URISyntaxException;
import java.util.Date;
import java.util.GregorianCalendar;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;


/**
 * API とのデータの送受信を行うクラスです。
 * 
 * @author $api_author Garoon Team@Cybozu$
 * @version $api_version ver 1.0.0$
 */
public class CBServiceClient {
	/** メッセージ有効期限の初期値 **/
    private static final Date DEFAULT_EXPIRED_TIME = new GregorianCalendar(2037, 11, 31).getTime();

    /** ユーザー名 **/
    private String username = "";
    
    /** パスワード **/
    private String password = "";
    
    /** ガルーンのURL **/
    private URI garoonURI;
    
    /** メッセージの作成日時 **/
    private Date createdTime = new GregorianCalendar().getTime();
    
    /** メッセージの有効期限 **/
    private Date expiredTime = DEFAULT_EXPIRED_TIME;

    /** スキーマ **/
    private String scheme = Constants.TRANSPORT_HTTP;

	private final ServiceClient serviceClient = new ServiceClient();
    private final OMFactory omFactory = OMAbstractFactory.getOMFactory();
    private final OMNamespace omNs = this.omFactory.createOMNamespace("http://wsdl.cybozu.co.jp/api/2008", "tns");

    /**
     * 新しいクライアントインスタンスを生成します。
     * @throws AxisFault サーバーと接続エラー
     */
    public CBServiceClient() throws AxisFault {
    }

    /**
     * ユーザーを設定します。<br />
     * ユーザー名、パスワード共に非nullである必要があります。
     * @param username ユーザー名
     * @param password パスワード
     */
    public void setUser(String username, String password) {
        if (username != null && password != null) {
            this.username = username;
            this.password = password;
        }
    }

    /**
     * メッセージの作成日時を設定します。<br />
     * null が渡された場合、値は無視されます。
     * @param createdTime 作成日時
     */
    public void setCreatedTime(Date createdTime) {
        if (createdTime != null)
            this.createdTime = createdTime;
    }

    /**
     * メッセージの有効期限を設定します。
     * null が渡された場合、値は無視されます。
     * @param expiredTime 有効期限
     */
    public void setExpiredTime(Date expiredTime) {
        if (expiredTime != null)
            this.expiredTime = expiredTime;
    }

    /**
     * スキーマを設定します。<br />
     * 例：http, https<br />
     * null が渡された場合、値は無視されます。
     * @param scheme スキーマ
     * @see org.apache.axis2.Constants
     */
    public void setScheme(String scheme) {
        if (scheme != null)
            this.scheme = scheme;
    }

    /**
     * ガルーンの URL を設定します。
     * null が渡された場合、値は無視されます。
     * 
     * @param uri URI
     */
    public void setGaroonURI(URI uri) {
        if (uri != null)
            this.garoonURI = uri;
    }

    /**
     * 受け取ったアクションを送信し、結果を受け取ります。
     * 
     * @param action SOAP アクション
     * @return org.apache.axiom.om.OMElement レスポンス
     * @throws AxisFault サーバーと接続エラー
     */
    public OMElement sendReceive(Action action) throws AxisFault {
        this.serviceClient.removeHeaders();

        // Set URL
        URI uri;
        try {
            uri = new URI(this.garoonURI.toString() + action.getAPIType().getPath());
        } catch (URISyntaxException e) {
            throw new AxisFault("Invalid URL:" + this.garoonURI.toString() + action.getAPIType().getPath());
        }

        // Add Header
        OMElement header = HeaderFactory.create(action.getActionName(), this.username, this.password, this.createdTime,
                this.expiredTime);
        this.serviceClient.addHeader(header);

        // Set Options
        Options options = OptionsFactory.create(uri, this.scheme, action.getActionName());
        this.serviceClient.setOptions(options);

        OMElement request = getRequest(action);
        OMElement result = this.serviceClient.sendReceive(request);

        return result;
    }

    private OMElement getRequest(Action action) {
        OMElement request = this.omFactory.createOMElement(action.getActionName(), this.omNs);
        OMElement parameters = action.getParameters();
        request.addChild(parameters);
        return request;
    }

    /**
     * アカウントファイルを元に設定を反映させます。<br />
     * このメソッドは次のメソッドで設定できる値を反映させます。<br />
     * {@link #setGaroonURI(URI)},{@link #setUser(String, String)},{@link #setScheme(String)},
     * {@link #setCreatedTime(Date)},{@link #setExpiredTime(Date)}
     * @param config ガルーンの設定情報
     */
    public void load(Config config) {
        setGaroonURI(config.getGaroonURL());
        setUser(config.getUsername(), config.getPassword());
        setScheme(config.getScheme());
        setCreatedTime(config.getCreatedTime());
        setExpiredTime(config.getExpiredTime());
    }

    /**
     * サーバーとのコネクションを切断します。<br />
     * コネクションを切断せずに、ひとつの CBServiceClient インスタンスを用いて複数回
     * APIとのやりとりを行うと org.apache.commons.httpclient.ConnectionPoolTimeoutException がスローされます。<br />
     * 
     */
    public void cleanupTransport()
    {
        try {
            this.serviceClient.cleanupTransport();
        } catch (AxisFault e) {
        }
    }
}
