package com.cybozu.garoon3.message;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;

import com.cybozu.garoon3.base.BaseChangeLogType;
import com.cybozu.garoon3.util.DateUtil;

/**
 * メッセージの情報を表すクラスです。<br />
 * message.xsd の ThreadType を表します。
 * 
 * @author $api_author Garoon Team@Cybozu$
 * @version $api_version ver 1.0.0$
 */
public class ThreadType {
    private final List<Addressee> addresseeList = new ArrayList<Addressee>();
    private final ThreadContent content = new ThreadContent();
    private final Map<Integer, Integer> followMap = new HashMap<Integer, Integer>();
    private final List<Integer> folderList = new ArrayList<Integer>();
    private BaseChangeLogType creator;
    private BaseChangeLogType modifier;
    private int id;
    private int version;
    private String subject;
    private boolean confirm = false;
    private Date snapshot;
    private boolean isDraft = false;

    /**
     * コンストラクタ
     */
    public ThreadType() {}
    /**
     * コンストラクタ
     * @param id メッセージID
     * @param version メッセージの更新情報
     * @param confirm メッセージの確認情報
     * @param subject メッセージの標題
     */
    public ThreadType(int id, int version, boolean confirm, String subject) {
        this.id = id;
        this.version = version;
        this.confirm = confirm;
        this.subject = subject;
    }

    /**
     * メッセージに宛先を追加します。
     * @param addressee 宛先
     */
    public void addAddressee(Addressee addressee){
        this.addresseeList.add(addressee);
    }

    /**
     * メッセージに本文を設定します。
     * @param body メッセージ本文
     */
    public void setBody(String body){
        this.content.setBody(body);
    }

    /**
     * メッセージに書式編集を使用した本文を設定します。
     * @param htmlBody 書式編集を使用したメッセージ本文
     */
    public void setHTMLBody(String htmlBody){
        this.content.setHTMLBody(htmlBody);
    }

    /**
     * メッセージにフォロー情報を追加します。
     * @param id フォローID
     * @param number フォロー番号
     */
    public void addFollow(int id, int number){
        this.followMap.put(id, number);
    }

    /**
     * メッセージが存在するフォルダIDを追加します。
     * @param id フォルダID
     */
    public void addFolder(int id){
        this.folderList.add(id);
    }

    /**
     * メッセージの作成者情報を設定します。
     * @param creator 作成者
     */
    public void setCreator(BaseChangeLogType creator){
        this.creator = new BaseChangeLogType(creator);
    }

    /**
     * メッセージの更新者情報を設定します。
     * @param modifier 更新者
     */
    public void setModifier(BaseChangeLogType modifier){
        this.modifier = new BaseChangeLogType(modifier);
    }

    /**
     * メッセージIDを設定します。
     * @param id メッセージID
     */
    public void setId(int id){
        this.id = id;
    }

    /**
     * メッセージのバージョン情報を設定します。
     * @param version バージョン情報
     */
    public void setVersion(int version){
        this.version = version;
    }

    /**
     * メッセージの標題を設定します。
     * @param subject 標題
     */
    public void setSubject(String subject){
        this.subject = subject;
    }

    /**
     * メッセージの閲覧状況を確認するか設定します。
     * @param confirm 閲覧状況を確認するか(true:確認する false:確認しない)
     */
    public void setConfirm(boolean confirm){
        this.confirm = confirm;
    }

    /**
     * メッセージのスナップショット日時を設定します。
     * @param snapshot スナップショット日時
     */
    public void setSnapshot(Date snapshot){
        this.snapshot = (Date) snapshot.clone();
    }

    /**
     * メッセージが下書きかどうか設定します。
     * @param isDraft 下書きかどうか(true:下書き false:下書きではない)
     */
    public void setIsDraft(boolean isDraft){
        this.isDraft = isDraft;
    }

    /**
     * メッセージに添付ファイルを追加します。
     * @param file 添付ファイル
     */
    public void addFile(AttachedFile file) {
        this.content.addFile(file);
    }

    /**
     * メッセージの標題を取得します。
     * @return
     */
    public String getSubject() {
        return this.subject;
    }
    /**
     * メッセージの内容を取得します。
     * @return
     */
    public ThreadContent getContent(){
        return this.content;
    }
    /**
     * メッセージの宛先を取得します。
     * @return
     */
    public List<Addressee> getAddressees() {
        return this.addresseeList;
    }

    /**
     * メッセージ情報を表すOMElementインスタンスを取得します。
     * @param name OMElementの要素名
     * @return OMElementインスタンス
     */
    public OMElement toOMElement(String name){
        OMFactory omFactory = OMAbstractFactory.getOMFactory();
        OMElement element = omFactory.createOMElement(name, null);

        for(Addressee addressee : this.addresseeList){
            element.addChild(addressee.toOMElement());
        }
        element.addChild(this.content.toOMElement());
        for(Map.Entry<Integer, Integer> e : this.followMap.entrySet()){
            OMElement follow = omFactory.createOMElement("follow", null);
            follow.addAttribute("id", String.valueOf(e.getKey()), null);
            follow.addAttribute("number", String.valueOf(e.getValue()), null);
            element.addChild(follow);
        }
        for(Integer id : this.folderList){
            OMElement folder = omFactory.createOMElement("folder", null);
            folder.addAttribute("id", String.valueOf(id), null);
            element.addChild(folder);
        }
        if(this.creator != null){
            element.addChild(this.creator.toOMElement());
        }
        if(this.modifier != null){
            element.addChild(this.modifier.toOMElement());
        }

        element.addAttribute("id", String.valueOf(this.id), null);
        element.addAttribute("version", String.valueOf(this.version), null);
        element.addAttribute("subject", this.subject, null);
        element.addAttribute("confirm", String.valueOf(this.confirm), null);
        if(this.snapshot != null){
            element.addAttribute("snapshot", DateUtil.dateToString(this.snapshot), null);
        }
        element.addAttribute("is_draft", String.valueOf(this.isDraft), null);

        return element;
    }
}
