package com.cybozu.garoon3.usecase;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.ParseException;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.CSVWriter;

import com.cybozu.garoon3.admin.AdminAddUserAccount;
import com.cybozu.garoon3.common.CBServiceClient;
import com.cybozu.garoon3.common.Config;
import com.cybozu.garoon3.usecase.util.CSVUtil;
import com.cybozu.garoon3.usecase.util.OptionsParser;

/**
 * ユーザーの追加を行うサンプルプログラムです。
 * 
 * @author $api_author Garoon Team@Cybozu$
 * @version $api_version ver 1.0.0$
 */
public class AddUser {
	/**
	 * サンプルプログラムを実行します。
	 * 
	 * @param args オプション
	 * @throws IOException ファイル入出力時に問題が発生した場合
	 * @throws MissingArgumentException 必要な引数が足りない場合
	 * @throws ParseException 引数にエラーがある場合
	 */
    public static void main(String[] args) throws IOException, MissingArgumentException, ParseException  {
        //Option check
        OptionsParser parser = new OptionsParser(args);
        if( !parser.hasAccountFileName() )
            throw new MissingArgumentException("Missing -g option");
        
        if( !parser.hasInputFileName() )
            throw new MissingArgumentException("Missing -i option");
        
        //Add users
        List<Map<String, String>> userList = getUserList( parser.getInputFileName() );

        //Create client
        CBServiceClient client = new CBServiceClient();
        Config config = new Config( parser.getAccountFileName() );
        client.load(config);

        List<Map<String, String>> addedUserList = new ArrayList<Map<String, String>>();
        
        //Add each user
        for(Map<String, String> user : userList)
        {
            String loginName   = user.get("login_name");
            String displayName = user.get("display_name");

            //Create action
            AdminAddUserAccount action = new AdminAddUserAccount(loginName, displayName);

            if(user.containsKey("password"))
                action.setPassword( user.get("password") );
            
            //Get result
            OMElement result = null;
            try {
                result = client.sendReceive( action );
            } catch(AxisFault e) {
                continue;
            } finally {
                client.cleanupTransport();
            }

            if( result != null )
                addedUserList.add( getAddedUserInfo(result) );
        }
        
        //Output file
        if(!parser.hasOutputFileName())
            return;
        
        writeFile(parser.getOutputFileName(), addedUserList);
    }
    
    /**
     * ユーザー情報をファイルにCSV形式で出力します。
     * 
     * @param filename 出力ファイル名
     * @param data ユーザーデータ
     * @throws IOException ファイル出力時のエラー
     */
    private static void writeFile(String filename, List<Map<String,String>> data) throws IOException
    {
        CSVWriter writer = new CSVWriter(new FileWriter(filename));

        //Write header
        writer.writeNext(new String[]{"userId", "login_name", "display_name", "password"});
        
        for(Map<String, String> addedUser : data)
        {
            List<String> line = new ArrayList<String>();
            line.add( addedUser.get("userId") );
            line.add( addedUser.get("login_name") );
            line.add( addedUser.get("display_name") );
            if(addedUser.containsKey("password_raw"))
                line.add( addedUser.get("password_raw") );
            else
            	line.add("");
            
            writer.writeNext(line.toArray(new String[]{}));
        }
        writer.close();
    }
    
    /**
     * APIのレスポンスを元に、追加されたユーザーの情報を取得します。
     * 
     * @param APIのレスポンス
     * @return Map<String, String> ユーザーデータリスト
     */
    @SuppressWarnings("unchecked")
	private static Map<String, String> getAddedUserInfo(OMElement result)
    {
        OMElement returns = result.getFirstElement();
        OMElement accountNode = returns.getFirstElement(); 
        Map<String, String> addedUser = new HashMap<String, String>();

        for(Iterator<OMElement> i = accountNode.getChildElements(); i.hasNext();) {
            OMElement e = i.next();
            addedUser.put(e.getLocalName(), e.getText());
        }

        return addedUser;
    }
    
    /**
     * 入力ファイルから追加するユーザーの情報を取得します。
     * 
     * @param inputFileName 入力ファイル名
     * @return List<Map<String,String>> ユーザーデータリスト
     * @throws IOException ファイル読み込み時のエラー
     */
    private static List<Map<String, String>> getUserList( String inputFileName ) throws IOException
    {
        //Read input csv file
        CSVReader reader = new CSVReader( new FileReader( inputFileName ) );
        List<Map<String, String>> users = CSVUtil.getColumnMapList(reader);
        reader.close();

        return users;

    }
}
