package com.cybozu.garoon3.usecase;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.axiom.om.OMElement;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.ParseException;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.CSVWriter;

import com.cybozu.garoon3.admin.AdminRemoveUsersByIds;
import com.cybozu.garoon3.common.CBServiceClient;
import com.cybozu.garoon3.common.Config;
import com.cybozu.garoon3.usecase.util.CSVUtil;
import com.cybozu.garoon3.usecase.util.OptionsParser;

/**
 * ユーザーの削除を行うサンプルプログラムです。
 * 
 * @author $api_author Garoon Team@Cybozu$
 * @version $api_version ver 1.0.0$
 */
public class RemoveUser {

	/**
	 * サンプルプログラムを実行します。
	 * 
	 * @param args オプション
	 * @throws IOException ファイル入出力時に問題が発生した場合
	 * @throws MissingArgumentException 必要な引数が足りない場合
	 * @throws ParseException 引数にエラーがある場合
	 */
    public static void main(String[] args) throws IOException, MissingArgumentException, ParseException {
        //Option check
        OptionsParser parser = new OptionsParser(args);
        if( !parser.hasAccountFileName() )
            throw new MissingArgumentException("Missing -g option");
        
        if( !parser.hasInputFileName() )
            throw new MissingArgumentException("Missing -i option");
        
        //Create action
        AdminRemoveUsersByIds action = new AdminRemoveUsersByIds();

        //Add users
        List<Integer> userIdList = getUserIDList( parser.getInputFileName() );
        for(Integer userId : userIdList)
            action.addUserID( userId );
        
        //Create client
        CBServiceClient client = new CBServiceClient();
        Config config = new Config( parser.getAccountFileName() );
        client.load(config);
        
        //Get result
        OMElement result = client.sendReceive( action );
        
        //Output file
        if(!parser.hasOutputFileName())
            return;

        //Parse result
        List<String> removedUserIds = getRemovedUserIDList(result);

        CSVWriter writer = new CSVWriter(new FileWriter( parser.getOutputFileName() ));
        writer.writeNext(new String[]{"user_id"});
        writer.writeNext(removedUserIds.toArray(new String[]{}));
        writer.close();
    }
    
    /**
     * APIのレスポンスを元に、削除されたユーザーIDのリストを生成します。
     * 
     * @param result APIのレスポンス
     * @return List<String> 削除されたユーザーID一覧
     */
    @SuppressWarnings("unchecked")
	private static List<String> getRemovedUserIDList(OMElement result)
    {
        List<String> removedUserIds = new ArrayList<String>();
        
        OMElement returns = result.getFirstElement();
        for(Iterator<OMElement> i = returns.getChildElements(); i.hasNext();)
        {
            OMElement e = i.next();
            if(e.getLocalName().equals("affected_userId"))
            {
                removedUserIds.add(e.getText().trim());
            }
        }

        return removedUserIds;
    }
    
    /**
     * 入力ファイルから削除するユーザーIDのリストを取得します。
     * 
     * @param inputFileName 入力ファイル名
     * @return List<Integer> 削除するユーザーID一覧
     * @throws IOException ファイル操作時のエラー
     */
    private static List<Integer> getUserIDList( String inputFileName ) throws IOException
    {
        //Read input csv file
        CSVReader reader = new CSVReader( new FileReader( inputFileName ) );
        List<String> userIdStringList = CSVUtil.getColumn(reader, "user_id");

        //Parse String to Integer
        List<Integer> userIdList = new ArrayList<Integer>();
        for(String id : userIdStringList){
            try{
                userIdList.add( Integer.valueOf(id) );
            }catch(Exception e){}
        }

        reader.close();
        return userIdList;
    }
}
