package com.cybozu.garoon3.usecase.util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import au.com.bytecode.opencsv.CSVReader;

/**
 * CSVで頻繁に利用される処理をまとめたユーティリティクラスです。<br />
 * 
 * @author $api_author Garoon Team@Cybozu$
 * @version $api_version ver 1.0.0$
 */
public class CSVUtil {
	
	/**
	 * 指定されたカラム名の列データを返します。<br />
	 * 一致するカラムが存在しない場合は空のリストを返します。
	 * 従って、このメソッドは null を返しません。
	 * 
	 * @param reader CSVリーダー
	 * @param columnName 取得するカラム名
	 * @return List<String> 列データ
	 */
    public static List<String> getColumn(CSVReader reader, String columnName)
    {
    	List<String[]> lines = null;
        try {
			lines = reader.readAll();
		} catch (IOException e) {
			return Collections.emptyList();
		}

		if(lines == null)
			return Collections.emptyList();
		
		//Find index
		int index = -1;
		String[] columnNames = lines.get(0);
		for(int i=0; i<columnNames.length; i++)
			if(columnName.equals(columnNames[i]))
				index = i;
		
		if( index == -1)
			return Collections.emptyList();

        //Remove Header Line
        lines.remove(0);

        List<String> result = new ArrayList<String>();
        for(String[] line : lines)
        {
        	result.add( removeQuotes(line[index]) );
        }
        return result;
    }
    
    /**
     * CSV に渡す文字列のクォーテーション文字をエスケープします。
     * @param input 入力文字列
     * @return 出力文字列
     */
    public static String escape(String input){
        return "\"" + input.replaceAll("\"", "\\\"\\\"") + "\"";
    }
    
    /**
     * CSVの全ての行のカラム名の各カラムの値のマップのリストを返します。<br />
     * 先頭のカラム名の行は処理されません。
     * このメソッドは null を返しません。
     * 
     * @param reader CSVリーダー
     * @return List<Map<String,String>> CSVデータ
     */
    public static List<Map<String, String>> getColumnMapList(CSVReader reader)
    {
    	List<String[]> lines = null;
        try {
			lines = reader.readAll();
		} catch (IOException e) {
			return Collections.emptyList();
		}

		if(lines == null)
			return Collections.emptyList();

		//Get Header Line
		String[] header = lines.get(0);
		
        //Remove Header Line
        lines.remove(0);

    	List<Map<String,String>> result = new ArrayList<Map<String,String>>();
        for(String[] line : lines)
        {
			Map<String, String> map = new HashMap<String, String>();
			
			for(int i=0; i<header.length; i++)
				map.put(removeQuotes(header[i]), removeQuotes(line[i]));

			result.add( map );
        }
        return result;
    }
    
    private static String removeQuotes(String input)
    {
    	return input.replaceAll("\"", "").trim();
    }
}
