package com.cybozu.garoon3.usecase.util;

import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

/**
 * コマンドライン引数で渡されたオプションをパースします。<br />
 * このクラスはサンプルプログラム専用のパーサーで、以下のオプションのみ解析を行います。<br />
 * -g アカウントファイル名<br />
 * -i 入力ファイル名<br />
 * -o 出力ファイル名<br />
 * <br />
 * このクラスはオプションの必須チェックは行いません。<br />
 * 必須チェックは各サンプルプログラムで行う必要があります。<br />
 * 
 * @author $api_author Garoon Team@Cybozu$
 * @version $api_version ver 1.0.0$
 */
public class OptionsParser {
    private static final String OPTION_KEY_INI_FILE     = "g";
    private static final String OPTION_KEY_INPUT_FILE   = "i";
    private static final String OPTION_KEY_OUTPUT_FILE  = "o";
    private CommandLine commandLine = null;

    /**
     * パースする対象のオプション配列を渡します。
     * 通常は main 関数の引数が渡されます。
     * @param args オプション配列
     * @throws ParseException 引数のエラー
     */
    public OptionsParser(String args[]) throws ParseException {
        //Parse command line option
        BasicParser parser = new BasicParser();
        Options options = new Options();
        options.addOption(OPTION_KEY_INI_FILE,       true, "setting file");
        options.addOption(OPTION_KEY_INPUT_FILE,     true, "input file");
        options.addOption(OPTION_KEY_OUTPUT_FILE,    true, "output file name");

        commandLine = parser.parse(options, args);
    }
    
    /**
     * -g オプションが存在するかどうかを返します。
     * @return -gオプションが存在する場合はtrue
     */
    public boolean hasAccountFileName()
    {
        return commandLine.hasOption( OPTION_KEY_INI_FILE );
    }
    
    /**
     * アカウントファイル名を返します。
     * -g オプションが存在しない場合は null.
     * @return java.lang.String アカウントファイル名
     */
    public String getAccountFileName()
    {
        return getValue( OPTION_KEY_INI_FILE );
    }
    
    /**
     * -i オプションが存在するかどうかを返します。
     * @return -iオプションが存在する場合はtrue
     */
    public boolean hasInputFileName()
    {
        return commandLine.hasOption( OPTION_KEY_INPUT_FILE );
    }
    
    /**
     * 入力ファイル名を返します。
     * -i オプションが存在しない場合は null.
     * @return java.lang.String 入力ファイル名
     */
    public String getInputFileName()
    {
        return getValue( OPTION_KEY_INPUT_FILE );
    }

    /**
     * -o オプションが存在するかどうかを返します。
     * @return -o オプションが存在する場合はtrue
     */
    public boolean hasOutputFileName()
    {
        return commandLine.hasOption( OPTION_KEY_OUTPUT_FILE );
    }
    
    /**
     * 出力ファイル名を返します。
     * -o オプションが指定されていない場合は null.
     * @return java.lang.String 出力ファイル名
     */
    public String getOutputFileName()
    {
        return getValue( OPTION_KEY_OUTPUT_FILE );
    }
    
    private String getValue(String key)
    {
        if( commandLine.hasOption(key) )
            return commandLine.getOptionValue(key);
        else
            return null;
    }
}
